{+--------------------------------------------------------------------------+
 | Expert:   Vcl FormAsComponent
 | Author:      Martin Waldenburg
 | Copyright    1997, all rights reserved.
 | Description: Expert for forms, which are components.
 | Version:     1.1
 | Status:      PostCardWare. Send me a Postcard and you may use it as you like.
 | DISCLAIMER: This is provided as is, expressly without a warranty of any kind.
 |             You use it at your own risc.
 +--------------------------------------------------------------------------+}

unit FoAsCoEx;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls, Forms,
  Dialogs, ExptIntf, ToolIntf, EditIntf, Windows, ExtCtrls, StdCtrls, Buttons,
  ComCtrls;

type

  TExpertForm = class(TForm)
    OKbtn: TButton;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    procedure OKClick(Sender: TObject);
  private
    { Private declarations }
    SourceBuffer: PChar;
    procedure WriteFormatedText(Stream: TStream; FormatedText: PChar; const Args: array of const);
    function DoFormCreation(const FormIdent: string): TForm;
    function CreateSource(const UnitIdent, FormIdent: string): TMemoryStream;
    function CreateForm(const FormIdent: string): TMemoryStream;
  public
    { Public declarations }
  end; { TExpertForm }

  TFormAsComponentExpert = class(TIExpert)
  private
    MenuItem: TIMenuItemIntf;
  protected
    procedure OnClick( Sender: TIMenuItemIntf); virtual;
  public
    constructor Create; virtual;
    destructor Destroy; override;
    function GetName: string; override;
    function GetAuthor: string; override;
    function GetStyle: TExpertStyle; override;
    function GetIDString: string; override;
  end;  { TFormAsComponentExpert }

  TIsControlToTrue = class(TComponent)
  private
    procedure SetIsControl(Value: Boolean);
    function GetDesigner: TDesigner;
  public
  published
    property FormIsControl: Boolean write SetIsControl;
  end;  { TIsControlToTrue }

procedure Register;

procedure FormExpert(ToolServices: TIToolServices);
var
  ExpertForm: TExpertForm;
  FormFileName: String;

implementation

uses Proxies, VirtIntf, IStreams;

{$R *.DFM}

const
  SourceBufferSize = 1024;

procedure Register;
begin
  RegisterLibraryExpert(TFormAsComponentExpert.Create);
  RegisterComponents('mw', [TIsControlToTrue]);
end;  { Register }

{ TFormAsComponentExpert code }
function TFormAsComponentExpert.GetName: String;
begin
  Result := 'FormAsComponentExpert'
end;

function TFormAsComponentExpert.GetAuthor: String;
begin
  Result := 'Martin_Waldenburg'; { author }
end;

function TFormAsComponentExpert.GetStyle: TExpertStyle;
begin
  Result := esAddIn;
end;

function TFormAsComponentExpert.GetIDString: String;
begin
  Result := 'private.FormAsComponentExpert';
end;

constructor TFormAsComponentExpert.Create;
var
  Main: TIMainMenuIntf;
  ReferenceMenuItem: TIMenuItemIntf;
  Menu: TIMenuItemIntf;
begin
  inherited Create;
  MenuItem := nil;
  if ToolServices <> nil then begin { I'm an expert! }
    Main := ToolServices.GetMainMenu;
    if Main <> nil then begin { we've got the main menu! }
      try 
        { add the menu of your choice }
        ReferenceMenuItem := Main.FindMenuItem('ComponentNewItem');
        if ReferenceMenuItem <> nil then
        try
          Menu := ReferenceMenuItem.GetParent;
          if Menu <> nil then
          try
            MenuItem := Menu.InsertItem(ReferenceMenuItem.GetIndex+1,
                              'FormAsComponent',
                              'FormAsComponentExpertItem','',
                              0,0,0,
                              [mfEnabled, mfVisible], OnClick);
          finally
            Menu.DestroyMenuItem;
          end;
        finally
          ReferenceMenuItem.DestroyMenuItem;
        end;
      finally
        Main.Free;
      end;
    end;
  end;
end;  { TFormAsComponentExpert.Create }

destructor TFormAsComponentExpert.Destroy;
begin
  if MenuItem <> nil then
    MenuItem.DestroyMenuItem;
  inherited Destroy;
end; { TFormAsComponentExpert.Destroy }

procedure TFormAsComponentExpert.OnClick( Sender: TIMenuItemIntf);
var
  ComponentInterface: TIComponentInterface;
  FormInterface: TIFormInterface;
  ModuleInterface: TIModuleInterface;
  PropValue: Boolean;
  FileName: String;
begin
  with TExpertForm.Create(Application) do
    try
      FormExpert(ToolServices);
      try
        ModuleInterface:= ToolServices.GetFormModuleInterface(FormFileName);
        FormInterface:= ModuleInterface.GetFormInterface;
        ComponentInterface := FormInterface.CreateComponent(NIL,'TIsControlToTrue',-1,-1,-1,-1);
        PropValue:= True;
        ComponentInterface.SetPropByName('FormIsControl', PropValue);
        ComponentInterface.Delete;
      finally
        ComponentInterface.Free;
        FormInterface.Free;
        ModuleInterface.Free;
      end;
    finally
      Free;
    end;
end;  { TFormAsComponentExpert.OnClick }

{ TIsControlToTrue }
function TIsControlToTrue.GetDesigner: TDesigner;
begin
    Result := (Owner as TForm).Designer;
end;  { GetDesigner }

procedure TIsControlToTrue.SetIsControl(Value: Boolean);
begin
  GetDesigner.IsControl := Value;
end;  { SetIsControl }

{ TFormExpert }
procedure TExpertForm.OKClick(Sender: TObject);
begin
  ModalResult := mrOK;
end;

function TExpertForm.DoFormCreation(const FormIdent: string): TForm;
begin
  Result := TForm.Create(nil);
  Proxies.CreateSubClass(Result, 'T' + FormIdent, TForm);
  with Result do
  begin
    BorderStyle := bsSizeable;
    Width := 400;
    Height := 300;
    Position := poScreenCenter;
    Name := FormIdent;
    Caption := FormIdent + '   This form is a Component';

    with Font do
    begin
      Name := 'MS Sans Serif';
      Size := 8;
    end;
  end;
end; { DoFormCreation }

procedure TExpertForm.WriteFormatedText(Stream: TStream; FormatedText: PChar;
  const Args: array of const);
begin
  if (Stream <> nil) and (SourceBuffer <> nil) then
  begin
    StrLFmt(SourceBuffer, SourceBufferSize, FormatedText, Args);
    Stream.Write(SourceBuffer[0], StrLen(SourceBuffer));
  end;
end;

function TExpertForm.CreateSource(const UnitIdent, FormIdent: string): TMemoryStream;
begin
  SourceBuffer := StrAlloc(SourceBufferSize);
  try
    Result := TMemoryStream.Create;
    try

      { unit header and uses clause }
      WriteFormatedText(Result,
        'unit %s;'#13#10#13#10 +
        'interface'#13#10#13#10 +
        'uses'#13#10 +
        '  SysUtils, Windows, Messages, Classes, Graphics, Controls,'#13#10 +
        '  StdCtrls, ExtCtrls, Forms;'#13#10#13#10, [UnitIdent]);

      { begin the class declaration }
      WriteFormatedText(Result,
        'type'#13#10 +
        '  T%s = class(TForm)'#13#10 +
        '  private'#13#10 +
        '    function RunTime: Boolean;'#13#10 +
        '  protected'#13#10 +
        '  public'#13#10 +
        '    constructor Create(AOwner: TComponent); override;'#13#10 +
        '    destructor Destroy; override;'#13#10 +
        '  published'#13#10 +
        '  end;'#13#10#13#10, [FormIdent]);

      WriteFormatedText(Result,
        'var'#13#10 +
        '  %s: T%s;'#13#10#13#10 +
        'procedure Register;'#13#10#13#10 +
        'implementation'#13#10#13#10 +
        '{$R *.DFM}'#13#10#13#10, [FormIdent, FormIdent]);

      WriteFormatedText(Result,
        'procedure Register;'#13#10 +
        'begin'#13#10 +
        '  RegisterComponents('#39'mw'#39', [T%s]);'#13#10 +
        'end;  { Register }'#13#10#13#10, [FormIdent]);

       WriteFormatedText(Result,
        'constructor T%s.Create(AOwner: TComponent);'#13#10 +
        'begin'#13#10 +
        '  inherited Create(AOwner);'#13#10 +
        '  ControlStyle := ControlStyle - [csAcceptsControls, csSetCaption];'#13#10 +
        '  Caption:='#39#39';'#13#10 +
        '  if not RunTime then'#13#10 +
        '    begin'#13#10 +
        '      Height:= 31;'#13#10 +
        '      Width:= 132;'#13#10 +
        '      WindowState:= wsMinimized;'#13#10 +
        '    end'#13#10 +
        '  else'#13#10 +
        '    begin'#13#10 +
        '  end;'#13#10 +
        'end;  { Create }'#13#10#13#10, [FormIdent]);

      WriteFormatedText(Result,
        'destructor T%s.Destroy;'#13#10 +
        'begin'#13#10 +
        '  Parent:= nil;'#13#10 +
        '  inherited Destroy;'#13#10 +
        '  end;  { Destroy }'#13#10#13#10, [FormIdent]);

      WriteFormatedText(Result,
        'function T%s.RunTime: Boolean;'#13#10 +
        'begin'#13#10 +
        '  RunTime := not (csDesigning in ComponentState);'#13#10 +
        'end;  { RunTime }'#13#10#13#10, [FormIdent]);

      WriteFormatedText(Result, 'end.'#13#10, [nil]);
      Result.Position := 0;

    except
      Result.Free;
      raise;
    end;

  finally
    StrDispose(SourceBuffer);
  end;
end;

function TExpertForm.CreateForm(const FormIdent: string): TMemoryStream;
var
  NewForm: TForm;
begin
  Result := nil;
  FormFileName:= FormIdent;
  NewForm := DoFormCreation(FormIdent);
  try
    Result := TMemoryStream.Create;
    Result.WriteComponentRes(FormIdent, NewForm);
    Result.Position := 0;
  finally
    NewForm.Free;
  end;
end;  { TExpertForm.CreateForm }

procedure FormExpert(ToolServices: TIToolServices);
var
  ExpForm: TExpertForm;
  ISourceStream, IFormStream: TIMemoryStream;
  UnitIdent, FormIdent: string;
  FileName: TFileName;
begin
  if ToolServices = nil then Exit;
  if ToolServices.GetNewModuleName(UnitIdent, FileName) then
  begin
    ExpForm := TExpertForm.Create(Application);
    try
      if ExpForm.ShowModal = mrOK then
      begin
        UnitIdent := LowerCase(UnitIdent);
        UnitIdent[1] := Upcase(UnitIdent[1]);
        FormIdent := 'Form' + Copy(UnitIdent, 5, 255);

        IFormStream := TIMemoryStream.Create(ExpForm.CreateForm(FormIdent));
        try
          IFormStream.AddRef;
          ISourceStream := TIMemoryStream.Create(ExpForm.CreateSource(UnitIdent,
            FormIdent));
          try
            ISourceStream.AddRef;
            ToolServices.CreateModule(FileName, ISourceStream, IFormStream,
              [cmAddToProject, cmShowSource, cmShowForm, cmUnNamed,
              cmMarkModified]);
          finally
            ISourceStream.OwnStream := True;
            ISourceStream.Free;
          end;

        finally
          IFormStream.OwnStream := True;
          IFormStream.Free;
        end;

      end;
    finally
      ExpForm.Free;
    end;
  end;
end;  { FormExpert }

end.

{ Martin Waldenburg
  Landaeckerstrasse 27
  71642 Ludwigsburg
  Germany }
